#!/usr/bin/env python
# vim: sts=4 sw=4

from libxml2pyxml import *
from xml.dom import getDOMImplementation
import libxml2

import sys

from ZSI.wstools.logging import setBasicLoggerDEBUG
setBasicLoggerDEBUG()

from zope.interface import classProvides, implements, Interface
from twisted.python import log

# ZSI imports
from ZSI import _get_element_nsuri_name, EvaluateException, ParseException
from ZSI import fault, ParsedSoap, SoapWriter

from ZSI.twisted.interfaces import HandlerChainInterface, CallbackChainInterface, DataHandler, DefaultHandlerChain

from soapclient import SOAPClient

class XMPPDataHandler:
    classProvides(HandlerChainInterface)

    @classmethod
    def processRequest(cls, input, **kw):
        return ParsedSoap(input, readerclass=libxml2Reader)

    @classmethod
    def processResponse(cls, output, **kw):
        sw = SoapWriter()
        sw.serialize(output)
        return sw

class XMPPCallbackHandler:
    classProvides(CallbackChainInterface)

    @classmethod
    def processRequest(cls, ps, **kw):
        """invokes callback that should return a response
        representing the SOAP response.
        ps -- ParsedSoap instance representing HTTP Body.
        """
        resource = kw['resource']
        method =  getattr(resource, 'soap_%s' %
                           _get_element_nsuri_name(ps.body_root)[-1])
                                              
        return method(ps)
    

class XMPPHandlerChainFactory:
    protocol = DefaultHandlerChain
    
    @classmethod
    def newInstance(cls):
        return cls.protocol(XMPPCallbackHandler, XMPPDataHandler, return_raw = True)
    
class SOAPContainer:
    factory = XMPPHandlerChainFactory
    
    def __init__(self, service, **kw):
        self.delegate = service
        self._l2_doc = libxml2.newDoc("1.0")
        
    def process(self, data):
        return pyxml2libxml(self._process(data), self._l2_doc)
        
    def _process(self, data):
        resource = self.delegate or self
        chain = self.factory.newInstance()
        try:
            result = chain.processRequest(data, resource=resource)
            soap = chain.processResponse(result, resource=resource)
        except Exception, ex:
            errsw = SoapWriter()
            fault.FaultFromException(ex, False, sys.exc_info()[2]).serialize(errsw)
            return errsw.dom.node
        
        return soap.dom.node

class XMPPContainer(SOAPContainer, SOAPClient):
    def __init__(self, jid, password, service, **kw):
        SOAPContainer.__init__(self, service, **kw)

        SOAPClient.__init__(self, jid, password, container = self, **kw)

    def serve(self):
        self.connect()
        try:
            self.loop(1)
        except KeyboardInterrupt:
            self.disconnect()
