#!/usr/bin/env python

import libxml2
import xml.dom

from ZSI.wstools.Namespaces import XMLNS

__all__ = ['libxml2pyxml', 'pyxml2libxml', 'libxml2Reader']

class libxml2Reader:
    """
    Conversion reader, creates dom.xml document from libxml2.xmlNode
    """
    def fromStream(self, node):
        doc = xml.dom.getDOMImplementation().createDocument(None, None, None)
        doc.appendChild(libxml2pyxml(node, doc))
        return doc

def libxml2pyxml(node, doc):
        if not isinstance(node, libxml2.xmlNode):
                raise TypeError, "Expected libxml2.xmlNode"
        return lx2xd(node, doc)

def pyxml2libxml(node, doc):
        return xd2lx(node, doc, {})

# libxml2 -> xml.dom
def lx2list(node):
        while node:
                yield node
                node = node.next

def lx2xd(node, doc):
        if node.type == "element":
                return lx2xd_elt(node, doc)
        elif node.type == "text":
                return doc.createTextNode(node.content)
        elif node.type == "comment":
                return doc.createComment(node.content)
        elif node.type == "cdata":
                return doc.createCDATASection(node.content)
        else:
                raise TypeError, "Unknown node type: %s" % node.type

def lx2xd_elt(node, doc):
        if node.type != "element":
                return
        ns = node.ns()
        if ns:
                if ns.name:
                        qname = "%s:%s" % (ns.name, node.name)
                else:
                        qname = node.name
                elt = doc.createElementNS(ns.content, qname)
        else:
                elt = doc.createElement(node.name)
        for n  in lx2list(node.children): elt.appendChild(lx2xd(n, doc))
        for a  in lx2list(node.properties):  lx2xd_attr(a, elt)
        for nd in node.nsDefs() or []:    lx2xd_ns(nd, elt)
        return elt

def lx2xd_attr(attr, newnode):
        if attr.type != "attribute":
                return
        ns = attr.ns()
        if ns:
                qname = "%s:%s" % (ns.name, attr.name)
                newnode.setAttributeNS(ns.content, qname, attr.content)
        else:
                newnode.setAttribute(attr.name, attr.content)
        
def lx2xd_ns(ns, newnode):
        if ns.type != "namespace":
                return
        if ns.name:
                newnode.setAttributeNS(XMLNS.BASE, 'xmlns:%s' %ns.name, ns.content)
        else:
                newnode.setAttributeNS(XMLNS.BASE, 'xmlns', ns.content)

def splitQName(qname):
        parts = qname.split(':')
        if len(parts) == 1:
                return None, qname
        elif len(parts) == 2:
                return parts
        else:
                raise TypeError, "%s is not valid QName" % qname 

# xml.dom -> libxml2
def xd2lx_elt(node, l2node, nss = {}):
        elt = l2node.newChild(None, node.localName, None)
        # Load namespaces
        lns = nss.copy()
        for a in node.attributes._attrs.values():
                if a.prefix == "xmlns":
                        ns = elt.newNs(a.value, a.localName)
                        lns[(a.value, a.localName)] = ns
                elif not a.prefix and a.localName == "xmlns":
                        ns = elt.newNs(a.value, None)
                        lns[(a.value, None)] = ns
        if node.namespaceURI:
                ns = lns[(node.namespaceURI, node.prefix)]
                elt.setNs(ns)
        for a in node.attributes._attrs.values():
                if not a.prefix:
                        if a.localName != "xmlns":
                                elt.setProp(a.localName, a.value)
                elif a.prefix != "xmlns":
                        elt.setPropNs(lns[(a.namespaceURI, a.prefix)], a.localName, a.value)

        for n in node.childNodes: xd2lx(n, elt, lns)
        return elt

def xd2lx(node, l2node, nss = {}):
        if node.nodeType == node.ELEMENT_NODE:
                return xd2lx_elt(node, l2node, nss)
        elif node.nodeType == node.TEXT_NODE:
                return l2node.addChild(l2node.doc.newDocText(node.data))
        elif node.nodeType == node.COMMENT_NODE:
                return l2node.addChild(l2node.doc.newDocComment(node.data))
        elif node.nodeType == node.CDATA_SECTION_NODE:
                return l2node.addChild(l2node.doc.newCDataBlock(node.data))
        else:
                raise TypeError, "Unknown node type: %s" % node.nodeType

