#!/usr/bin/python -u
# vi: sts=4 et sw=4
#

from ZSI.wstools.Namespaces import SOAP

from pyxmpp.all import Iq,Presence,Message,StreamError
from pyxmpp.jabber.client import JabberClient

from xmpputils import obj2jid

class _SOAP(SOAP):
    # in xep SOAP-ENV is defined as w3.org
    #ENV = "http://www.w3.org/2003/05/soap-envelope"
    pass

class SOAPClient(JabberClient):
    """Simple bot (client) example. Uses `pyxmpp.jabber.client.JabberClient`
    class as base. That class provides basic stream setup (including
    authentication) and Service Discovery server. It also does server address
    and port discovery based on the JID provided."""
    def __init__(self, jid, password, container = None):
        # setup client with provided connection information
        # and identity data
        JabberClient.__init__(self, obj2jid(jid, "soap"), password,
                disco_name="pyXMPP ZSI bot", 
                disco_category='automation',
                disco_type="soap")

        # register features to be announced via Service Discovery
        self.disco_info.add_feature("jabber:iq:version")
        self.disco_info.add_feature("http://jabber.org/protocol/soap")

        self.container = container 

    def session_started(self):
        JabberClient.session_started(self)

        # set up handlers for supported <iq/> queries
        self.stream.set_iq_get_handler("query","jabber:iq:version",self.get_version)
        self.stream.set_iq_set_handler("Envelope",_SOAP.ENV,self.soap)

    def get_version(self,iq):
        """Handler for jabber:iq:version queries.

        jabber:iq:version queries are not supported directly by PyXMPP, so the
        XML node is accessed directly through the libxml2 API.  This should be
        used very carefully!"""
        iq=iq.make_result_response()
        q=iq.new_query("jabber:iq:version")
        q.newTextChild(q.ns(),"name","SOAP bot")
        q.newTextChild(q.ns(),"version","0.0.1")
        self.stream.send(iq)
        return True

    def soap(self,iq):
        container = self.container or self
        ret = container.process(iq.get_node().children)

        riq = iq.make_result_response()

        print ret
        riq.set_content(ret)
        self.stream.send(riq)
        return True

