#!/usr/bin/env python
# vi: sts=4 sw=4

from ZSI.client import Binding, ParsedSoap

from soapclient import SOAPClient
from libxml2pyxml import libxml2Reader

from pyxmpp.jabber.client import JabberClient
from pyxmpp.all import Iq
import pyxmpp.xmlextra
from xmpputils import obj2jid

from httplib import HTTPConnection

import libxml2
import random

class NullHTTPConnection(HTTPConnection):
    def __init__(self, arg1, arg2, **kw): pass
    def connect(self): pass

class ClientSOAPClient(JabberClient):
    def __init__(self, jid, password, **kw):

        JabberClient.__init__(self, obj2jid(jid, 'soap-client'), password, **kw)
        self._replies = {}

    def soap_err(self, iq):
        print "---------------- Error -------------------"
        self._replies[iq.get_id()] = iq.copy()

    def soap_res(self, iq):
        print "---------------- Reply -------------------"
        self._replies[iq.get_id()] = iq.copy()

    def soap_send(self, iq):
        def nuke(arg1, arg2):
            self.soap_timeout(self, iq.get_id())
        self._replies[iq.get_id()] = None
        self.stream.set_response_handlers(iq, self.soap_res, self.soap_err, nuke)
        self.stream.send(iq)
        self.stream.loop_iter(0)

    def soap_loop(self, id, to = 1):
        while self._replies.has_key(id):
            if self._replies[id] is not None:
                iq = self._replies[id]
                if iq.get_type() == "error":
                    raise TypeError("XMPP call failed: %s" % str(iq.get_node()))
                return iq.get_node().children
            self.stream.loop_iter(to)
        return None
        

class XMPPBinding(Binding):
    def __init__(self, jid, password, **kw):
        kw.setdefault('transport', NullHTTPConnection)
        Binding.__init__(self, **kw)
        self._xmpp = ClientSOAPClient(jid, password)

        self._xmpp.connect()
        self._xmpp.stream.loop_iter(1)
        self._last_id = None

    def SendSOAPData(self, soapdata, url, soapaction, **kw):
        if url[:5] == "xmpp:":
            url = url[5:]

        xml = libxml2.parseDoc(soapdata)
        print xml.children

        iq = Iq(to_jid=obj2jid(url, "soap"), stanza_type = "set", stanza_id = str(random.random()))
        iq.get_node().addChild(xml.children)
        self._last_id = iq.get_id()
        self._xmpp.soap_send(iq)

        # Clear prior receive state.
        self.data, self.ps = None, None

    def ReceiveSOAP(self, **kw):
        '''Get back a SOAP message.
        '''
        if self.ps: return self.ps
        if not self._last_id:
            raise TypeError("No request sent")

        self.data = self._xmpp.soap_loop(self._last_id)
        self.ps = ParsedSoap(self.data, 
                        readerclass=libxml2Reader,
                        encodingStyle=kw.get('encodingStyle'))

        if self.sig_handler is not None:
            self.sig_handler.verify(self.ps)

        return self.ps
        
